#!/usr/bin/python
"""pydicfunc.cgi

  Author: tgz <tgz@users.sourceforge.jp>
  Version: 2.0
  Last Modified: 2004/01/21

  This program is based on GPL.

"""
import cgi
import ConfigParser, os, re, string
import ndtplib


class NDTPSearcher:
    """ Super """
    def __init__(self, server, configfile):
        self.server = server
        self.NDTP = ndtplib.NDTP(self.server)
        self.version, self.dictdict = self.NDTP.version(), self.NDTP.listdic()
        self.NDTP.quit()
        self.word = ''
        self.result = {}
        self.current_dic = 0
        c = ConfigParser.ConfigParser()
        c.read(configfile)
        self.bitmappath = c.get('common', 'cachepath')
        self.bitmapsize = int(c.get('common', 'bitmapsize'))

    def search(self, word):
        self.word = word
        self.NDTP.connect(self.server)
        for num in self.dictdict.keys():
            self.current_dic = num
            self.NDTP.setdic(self.current_dic)
            if  len(self.NDTP.listbitmapsize()) > 0:
                self.NDTP.setbitmapsize(self.bitmapsize)
            self.result[num] = self.NDTP.ask(self.word)
        self.NDTP.quit()

    def show(self):
        self.show_header(self.word)
        self.show_main()
        self.show_footer()

    def show_header(self, title):
        print 'Content-type: text/html'
        print ''
        print '<html>'
        print '<head>'
        if self.word != None:
            print '<title>Result of \"%s\"</title>' % title
        print '</head>'
        print '<body>'
        print '<h1>Result of \"%s\"</h1>' % title
        print '<hr>'

    def convert_bitmap(self, str):
        prog = re.compile('(?P<gaiji><gaiji:[0-9a-zA-Z^>:]+>)')
        offset_pre, offset_post = len('<gaiji:'), int('-%d' % len('>'))
        cachepath = os.path.join(self.bitmappath, '%s' % self.current_dic, \
'%s' % self.bitmapsize)

        appendix = prog.search(str)

        while appendix != None:
            bitmapname = appendix.group('gaiji')[offset_pre:offset_post]
            src = os.path.join(cachepath, '%s.xbm' % bitmapname)
            str = string.replace(str, appendix.group('gaiji'), \
'<img src=\"%s\" alt=\"\">' % src)
            bitmapname = ''

            appendix = prog.search(str)

        return str

    def show_main(self):
        dicts = self.result.keys()
        dicts.sort()
        for num in dicts:
            print '<h2>%s [%d hits]</h2>' % (self.dictdict[num], \
len(self.result[num].keys()))
            print '<ol>'
            hits = self.result[num].keys()
            hits.sort()
            for hit in hits:
                print '<li><a href=\"/cgi-bin/pydicfunc.cgi?server=%s&num=%s&\
header=%s\">%s</a></li>' % (self.server, num, self.result[num][hit], \
self.convert_bitmap(hit))
            print '</ol>'

    def show_footer(self):
        print '<hr>'
        print '%s' % self.version
        print '</body>'
        print '</html>'

class NDTPDict(NDTPSearcher):
    def __init__(self, server, configfile, dicts):
        NDTPSearcher.__init__(self, server, configfile)
        self.dicts = dicts
        self.header = ''

    def search(self, header):
        self.header = header
        self.NDTP.connect(self.server)
        if len(self.dicts) == 1:
            num = self.dicts[0]
            self.current_dic = num
            self.NDTP.setdic(self.current_dic)
            if  len(self.NDTP.listbitmapsize()) > 0:
                self.NDTP.setbitmapsize(16)
            self.result[self.header] = self.NDTP.getmain(self.header)
        self.NDTP.quit()

    def show(self):
        self.show_header(self.header)
        self.show_main()
        self.show_footer()

    def enable_link(self, str):
        prog = re.compile('(?P<link><[0-9a-zA-Z^>]+:[0-9a-zA-Z^>]+>)')
        link = prog.search(str)
        while link != None:
            str = string.replace(str, link.group(), \
'<a href=\"/cgi-bin/pydicfunc.cgi?server=%s&num=%s&header=%s\">%s</a>' % \
(self.server, self.dicts[0], link.group('link')[1:-1], '(link)'))
            link = prog.search(str)
        return str

    def show_main(self):
        self.result[self.header] = self.convert_bitmap(self.result[self.header])
        self.result[self.header] = self.enable_link(self.result[self.header])
        self.result[self.header] = string.replace(self.result[self.header], \
'\n', '<br>\n')

        print '<h2>%s</h2>' % (self.dictdict[self.dicts[0]])
        print self.result[self.header]


def show_error(message):
    print '<html>'
    print '<head>'
    print '<body>'
    print '<h1>Error</h1><br>'
    print '%s<br>' % message
    print '</body>'
    print '</html>'

def main():
    try:
        configfile = os.path.join(os.environ['HOME'], '.pydic.conf')
        dicts = []
        form = cgi.FieldStorage()
        if form.has_key('server'):
            server = form['server'].value
        if form.has_key('num'):
            for i in  string.split(form['num'].value, ','):
                dicts.append(int(i))

        if form.has_key('word'):
            word = form['word'].value
            n2 = NDTPSearcher(server, configfile)
            n2.search(word)
            n2.show()
        elif form.has_key('header'):
            header = form['header'].value
            n3 = NDTPDict(server, configfile, dicts)
            n3.search(header)
            n3.show()

    except Exception, message:
        show_error(message)

if __name__=='__main__':
    main()
